#include"serial.h"

int fd;
int gmqttcount=0;//计算在第一次连接后规定时间是否获得消息
char lat[20], lat_dir, lon[20], lon_dir;

int init_serial(const char *port, int baudrate,speed_t baud) 
{
    int fd = open(port, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1) {
        perror("open serial port");
        return -1;
    }

    struct termios options;
    tcgetattr(fd, &options);

    // 设置波特率
    cfsetispeed(&options, baud);
    cfsetospeed(&options, baud);

    // 配置串口
    options.c_cflag &= ~PARENB; // 无校验
    options.c_cflag &= ~CSTOPB; // 1停止位
    options.c_cflag &= ~CSIZE;  // 清除数据位设置
    options.c_cflag |= CS8;     // 8数据位
    options.c_cflag |= CLOCAL | CREAD; // 忽略调制解调器状态，开启接收器

    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // 原始模式
    options.c_iflag &= ~(IXON | IXOFF | IXANY);         // 禁用流控
    options.c_oflag &= ~OPOST;                         // 原始输出

    tcsetattr(fd, TCSANOW, &options);
    return fd;
}


// 将NMEA格式的经纬度转换为十进制度
/*
double convert_to_decimal(const char *coordinate, char direction) {
    double degrees, minutes;
    sscanf(coordinate, "%15lf", &degrees);
    minutes=degrees%100;
    degrees=int(degrees/100);
    degrees += minutes / 60.0;
    if (direction == 'S' || direction == 'W') {
        degrees = -degrees;
    }
    return degrees;
}
*/
// 解析$GNRMC语句并提取经纬度
void parse_gnrmc(const char *nmea_sentence) {
    
    char time[11], status;
    if (sscanf(nmea_sentence, "$GNRMC,%10[^,],%c,%[^,],%c,%[^,],%c", time, &status, lat, &lat_dir, lon, &lon_dir) == 6) 
    {
        if (status == 'A') 
        { // 定位有效
            //double latitude =   convert_to_decimal(lat, lat_dir);
            //double longitude =  convert_to_decimal(lon, lon_dir);

            printf("Time (UTC): %s\n", time);
            printf("Latitude: %s\n", lat);
            printf("Longitude: %s\n", lon);
            //mqtt_wirte(lat);
            //mqtt_wirte(lon);
        } 
        else 
        {
            printf("GNRMC data is invalid (status: %c)\n", status);
        }
    } 
    else 
    {
        printf("Failed to parse GNRMC sentence.\n");
    }
}



void serial_Receive() {
    char buffer[BUFFER_SIZE];
    char nmea_sentence[BUFFER_SIZE];
    int index=0;
    //mosq = mosquitto_new(CLIENT_ID, true, NULL);
    while (1) 
    { 
        int bytes_read = read(fd, buffer, BUFFER_SIZE-1);
        if (bytes_read > 0) 
        {
            buffer[bytes_read] = '\0';
            for (int i = 0; i < bytes_read; i++) 
            {
                if (buffer[i] == '$') 
                {
                    index = 0; // 新语句开始
                }
                if (index < BUFFER_SIZE - 1) 
                {
                    nmea_sentence[index++] = buffer[i];
                    nmea_sentence[index] = '\0';
                }
                if (buffer[i] == '\n') 
                { // 一条语句结束
                    if (strstr(nmea_sentence, "$GNRMC")) 
                    {
                        parse_gnrmc(nmea_sentence); // 解析$GNRMC语句
                        
                    }
                }
            }   
        }
    mqtt_wirte();
    sleep(3);
    gmqttcount++;
    }
}



int serial_Init()
{
    fd = init_serial(SERIAL_PORT, 9600,B9600);
    if (fd == -1) return -1;
    printf("Serial communication started...\n");
    
}
